;+
; CUBEOBJECT::INTEGRATEZEROCURVE
;
; Intent:
;        Integrate a curve of Bx = By = 0 across the cube
;
; History:
;        CB, October 2006
;-
function cubeObject::IntegrateZeroCurve, q, dir=dir

x = q->getpos()
b = self->calcField(x)
curve = [x,b]
h = 0.5
repeat begin
    count = 0
    repeat begin
        dxd = self->RKZstep(x, h, dir)
        dx1 = self->RKZstep(x, 0.5*h, dir)
        dx  = dx1 + self->RKZstep(x + dx1, 0.5 * h, dir)
        
        wf = where(finite(dx), nwf)
        if nwf ne 3 then begin
            ;stop, curve
            return, curve
        endif 
        ok = compareRK(dxd, dx, h)
	count = count + 1
    endrep until ok ne 0 or count gt 10
    x = x + dx
    x = self->correctx(x, dx, b=b)
    curve = [[curve],[x,b]]
    if self->isinbox(x) eq 0 then done = 1 else done = 0
    if n_elements(curve[0,*]) gt 5000 then done = 1
    
endrep until done eq 1
;stop, curve
return, curve

end    


function cubeObject::IntegrateZeroCurve_old, q, dir = dir

x = q->getpos()
b = self->calcField(x)
curve = [x,b]
h = 0.5
repeat begin
    db = self->calcDerivs(x)
    wnf = where(finite(db) eq 0, nwnf)
    if nwnf eq 0 then begin
        xp = normalise(crossp(db[*,0], db[*,1]))
        dx1 = dir * h * xp
        dx2 = dir * h * xp * 2.0
        db1 = self->calcderivs(x + dx1)
        wnf2 = where(finite(db1) eq 0, nwnf2)
        if nwnf2 eq 0 then begin
            xp1 = normalise(crossp(db1[*,0], db1[*,1]))
            dx  = dx1 + dir * h * xp1
            d =sqrt(total((dx - dx2)^2))
            if finite(d) eq 0 then stop
            if finite(h) eq 0 then stop
            if d  gt h * 0.1 then begin
                h = (h * 0.9) > 1.e-1
            endif
            if d lt h * 1.e-3 then begin
                h = (h * 1.1) < 1.5
            endif
            x = x + dx

            b = self->calcfield(x)
            ; let's take a newton-step towards b_perp = 0
            if total(b[0:1]^2) gt 1.e-4 then begin

                j = self->calcderivs(x)
                ji = invert(j[0:1,0:1])
                dx = - ji ## b[0:1]
                dx = [dx[0],dx[1],0.]
                if total(dx^2) gt 0.01 then dx = normalise(dx) * 0.1
                
                x = x + [dx[0],dx[1],0.]
                b = self->calcfield(x)
            endif
            curve = [[curve],[x,b]]

        endif
    endif 
    

endrep until self->isInBox(x) eq 0 or n_elements(curve[0,*]) gt 5000 or nwnf gt 0 or nwnf2 gt 0
stop, curve
return, curve

end
